/*
 * optionscontroller.js
 *
 *  Created on: 08.02.2016
 *      Author: Holger Dahm
 *
 *  Note: adpat options to your app needs and set correct settings dialog URL
 */

// URL to settings page
var mobileToGo = "https://www.mobile-to-go.de/index.php?id=182";

// app controller used in this module
// as well as all info providers for
// json or xml request, caching as well as message to pebble send operations
// Note: call setAppController() before doing anything!
var appcontroller;

function setAppController(controller) {
	appcontroller = controller;
}



// app options
var OPTIONS_ID           = 'options';

// workaround to have changing setting dialog calls
var counter = 0;

var options = {
        // Basic Configuration
        "color" : 3,
        "language": 4,
        "activity": 0,
        "nightActivity": 0,
        "reload": 20,

        // Service Configuration
        "notification": 60,
        "vibration": 0,
        "weather": 1,
        "calendar": 1,

        // InfoPanel Configuration
        "accelerator": 0,
        "providerSwitchDelay": 60,

        // Weather Configuration
        "fahrenheit": 0,
        "inch": 0,
        "locationName": "Friedrichshafen",
        "lid": "18225472",
        "fixLocation": 0,

        // Calendar Configuration
        "calSundayMode": 0,

        // GPS Configuration
        "gpsaccuracy": 1,
        "gpstimeout": 10,
        "gpsage": 20,

        // Version Management
        "version": 12,

        // Licence Management
        "endtime": 0
     };



/*
 * clear options causing problems e.g. to show iPhone settings dialog
 */

function clearWrongOptions() {
	// nothing remains to do
}

/*
 * push options to app message queue
 */

function pushOptionsToAppmessageQueue() {
    // send configuration to pebble
    appcontroller.appMessageQueue.push({
        "message" : {
        	"settingsContent": 	options.vibration + "|" +
					            options.notification + "|" +
					            options.providerSwitchDelay + "|" +
					            options.accelerator + "|" +
					            options.activity + "|" +
					            options.nightActivity + "|" +
					            options.calSundayMode + "|" +
					            options.color + "|" +
					            options.language + "|" +
					            options.weather + "|" +
					            options.calendar + "|" +
					            options.reload + "|" +
					            options.endtime + "|e"
        }
    });
}

/*
 * get call settings dialog URL
 */

function getSettingsURL() {
	var url = mobileToGo + "&appconfigurationmanager[platform]=" + appcontroller.pebblehelpers.getPebblePlatform()
	                     + ( hasLicenceKey() ? "&appconfigurationmanager[licenceKey]=" + getLicenceKey() : "" )
	                     + "&appconfigurationmanager[accountToken]=" + Pebble.getAccountToken()
	                     + "&appconfigurationmanager[version]=" + options.version
	                     + "&workaround=" + (counter++);

    //console.log(url);

    return url;
}

/*
 * licence management operations
 */

function getLicenceKey() {
	return options.licenceKey;
}

function hasLicenceKey() {
	var licenceKey = getLicenceKey();
	return typeof licenceKey != "undefined" && licenceKey  != null && licenceKey.length > 1;
}

function isLicenceExpired() {
	var now = Math.round(new Date().getTime() / 1000);
    return options.endtime < Math.round(now / (60*60*24));
}



// option management

/*
 * clear all cached options
 */

function clearCache() {
    localStorage.removeItem(OPTIONS_ID);
}

/*
 * change single option value
 */

function setOption(name, value) {
    options[name] = value;

    localStorage.setItem(OPTIONS_ID, JSON.stringify(options));
    //console.log("*** changed option '" + name + "' to '" + value + "'");
}

/*
 * change all option values
 */

function setOptions(jsonOptions, infoproviders) {
    // determine transient properties, e.g. the provider count

	// nothing to do

    // store options
    localStorage.setItem(OPTIONS_ID, JSON.stringify(jsonOptions));
    //console.log("*** changed all options: " + JSON.stringify(jsonOptions));

    initialize(infoproviders);
}

/*
 * get options from remote DB
 */
function getRemoteOptions(infoproviders, callbackFunction) {
    var url = getSettingsURL()
	          + "&appconfigurationmanager[CONTENT_ACTION]=GET_APPCONFIGURATION"
	          + "&type=427";
	//console.log(url);

    appcontroller.startJSONRequest(url,
        function(result) {
    	    if(typeof result.error != "undefined") {
	    		console.log("Got licence error! " + result.error);

    	    	if (typeof result.endtime != "undefined") {
    	    		setOption("endtime", result.endtime);
    	    	}
    	    	else {
    	    		clearCache(); // clear options
    	    	}

                initialize(infoproviders);
    	    }
    	    else if (typeof result.licenceKey != "undefined") {
                // we got option JSON result
            	setOptions(result, infoproviders);
            }

    		console.log("Licence is " + (isLicenceExpired() ?  "expired!" : "valid."));

            callbackFunction();
        },
        function() {
            console.log("ERROR: Can't request remote options!");

            initialize(infoproviders);

            callbackFunction();
        }
    );
}

/*
 * initialize local options as well as options used in appcontroller and
 * all infoproviders
 */

function initialize(infoproviders) {
    var newOptions = JSON.parse(localStorage.getItem(OPTIONS_ID));

    // change options
    // currently nothing remains to do

    options = appcontroller.pebblehelpers.mergeArrays(options, newOptions);
    //console.log("*** initialize options: " + JSON.stringify(options));

    clearWrongOptions();

	appcontroller.setOptions(options);
    infoproviders.setOptions(options);
    //console.log('loading options: ' + JSON.stringify(options));
}



/*
 * interface
 */

module.exports = {
		// set app controller used in this module
		setAppController : setAppController,

		// push options to app message queue
		pushOptionsToAppmessageQueue : pushOptionsToAppmessageQueue,

		// clear all cached options
		clearCache : clearCache,

		// get call settings dialog URL
		getSettingsURL : getSettingsURL,

		// request options stored in DB
		getRemoteOptions : getRemoteOptions,

		// change all option values
		setOptions : setOptions,

		// check if licence is expired or not
		isLicenceExpired : isLicenceExpired,

		// initialize local options as well as options used in appcontroller and all infoproviders
		initialize : initialize
};
