/*
 * weatherinfoprovider.js
 *
 *  Created on: 08.02.2016
 *      Author: Holger Dahm
 *
 * Note: adapt this to your specific needs
 */

// (default) options used in this infoprovider
// Note: call setOptions() before doing anything!
var options;

function setOptions(values) {
	options = values;
}

// initialize appcontroller for
// json or xml request, caching as well as message to pebble send operations
// Note: call setAppController() before doing anything!
var appcontroller;

function setAppController(controller) {
	appcontroller = controller;
}

var WEATHER_ID           = 'weather';

/*
 * generic module interace
 */

module.exports = {
		// set (default) options used in this module
		setOptions : setOptions,

		// set app controller used in this module
		setAppController : setAppController,

		// IDs used for caching module infos
		WEATHER_ID : WEATHER_ID,
};



// module properties

var weatherIcon = new Object();

weatherIcon["\\d\\d\\d\\d\\d\\d1"]      = { icon : 3 }; // cold.png
weatherIcon["999997\\d"]                = { icon : 7 }; //  fog.png
weatherIcon["\\d\\d9997\\d"]            = { icon : 7 }; //  fog.png
weatherIcon["9999992"]                  = { icon : 2 }; //  wind.png
weatherIcon["\\d\\d9998\\d"]            = { icon : 2 }; //  wind.png
weatherIcon["\\d\\d\\d\\d\\d\\d2"]      = { icon : 2 }; //  wind.png
weatherIcon["\\d9\\d1\\d\\d\\d"]        = { icon : 9 }; //  snow.png
weatherIcon["\\d9\\d3\\d\\d\\d"]        = { icon : 9 }; //  snow.png
weatherIcon["\\d9\\d5\\d\\d\\d"]        = { icon : 9 }; //  snow.png
weatherIcon["\\d\\d1999\\d"]            = { icon : 17 }; //  drizzle.png
weatherIcon["\\d\\d2999\\d"]            = { icon : 17 }; //  drizzle.png
weatherIcon["\\d\\d3999\\d"]            = { icon : 8 }; // rain.png
weatherIcon["\\d\\d4999\\d"]            = { icon : 8 }; // rain.png
weatherIcon["\\d\\d5999\\d"]            = { icon : 8 }; // rain.png
weatherIcon["\\d\\d9199\\d"]            = { icon : 10 }; // sleet.png
weatherIcon["\\d\\d9299\\d"]            = { icon : 10 }; // sleet.png
weatherIcon["\\d\\d9399\\d"]            = { icon : 11 }; // snow-sleet.png
weatherIcon["\\d\\d9499\\d"]            = { icon : 11 }; // snow-sleet.png
weatherIcon["\\d\\d9599\\d"]            = { icon : 11 }; // snow-sleet.png
weatherIcon["\\d\\d9919\\d"]            = { icon : 4 }; // hail.png
weatherIcon["\\d\\d9929\\d"]            = { icon : 4 }; // hail.png
weatherIcon["\\d\\d9939\\d"]            = { icon : 4 }; // hail.png
weatherIcon["\\d\\d9949\\d"]            = { icon : 4 }; // hail.png
weatherIcon["\\d\\d9959\\d"]            = { icon : 4 }; // hail.png
weatherIcon["\\d\\d1199\\d"]            = { icon : 12 }; // rain-sleet.png
weatherIcon["\\d\\d2299\\d"]            = { icon : 12 }; // rain-sleet.png
weatherIcon["\\d\\d3399\\d"]            = { icon : 13 }; // rain-snow.png
weatherIcon["\\d\\d4499\\d"]            = { icon : 13 }; // rain-snow.png
weatherIcon["\\d\\d5599\\d"]            = { icon : 13 }; // rain-snow.png
weatherIcon["\\d\\d9191\\d"]            = { icon : 16 }; // thunder.png
weatherIcon["\\d\\d9491\\d"]            = { icon : 16 }; // thunder.png
weatherIcon["\\d\\d9991\\d"]            = { icon : 16 }; // thunder.png
weatherIcon["\\d\\d9993\\d"]            = { icon : 16 }; // thunder.png
weatherIcon["\\d\\d9995\\d"]            = { icon : 16 }; // thunder.png
weatherIcon["\\d\\d2291\\d"]            = { icon : 14 }; // rain-thunder.png
weatherIcon["\\d\\d4491\\d"]            = { icon : 14 }; // rain-thunder.png
weatherIcon["\\d\\d1991\\d"]            = { icon : 14 }; // rain-thunder.png
weatherIcon["\\d\\d2991\\d"]            = { icon : 14 }; // rain-thunder.png
weatherIcon["\\d\\d4991\\d"]            = { icon : 14 }; // rain-thunder.png
weatherIcon["\\d\\d9992\\d"]            = { icon : 14 }; // rain-thunder.png
weatherIcon["\\d\\d9994\\d"]            = { icon : 14 }; // rain-thunder.png
weatherIcon["11\\d\\d\\d\\d\\d"]        = { icon : 5 }; // partly-cloudy-day.png
weatherIcon["12\\d\\d\\d\\d\\d"]        = { icon : 5 }; // partly-cloudy-day.png
weatherIcon["13\\d\\d\\d\\d\\d"]        = { icon : 5 }; // partly-cloudy-day.png
weatherIcon["21\\d\\d\\d\\d\\d"]        = { icon : 6 }; // partly-cloudy-night.png
weatherIcon["22\\d\\d\\d\\d\\d"]        = { icon : 6 }; // partly-cloudy-night.png
weatherIcon["23\\d\\d\\d\\d\\d"]        = { icon : 6 }; // partly-cloudy-night.png
weatherIcon["1\\d\\d\\d\\d\\d\\d"]      = { icon : 0 }; // clear-day.png
weatherIcon["2\\d\\d\\d\\d\\d\\d"]      = { icon : 1 }; // clear-night.png
weatherIcon["\\d1\\d\\d\\d\\d\\d"]      = { icon : 15 }; // cloudy.png
weatherIcon["\\d2\\d\\d\\d\\d\\d"]      = { icon : 15 }; // cloudy.png
weatherIcon["\\d3\\d\\d\\d\\d\\d"]      = { icon : 15 }; // cloudy.png
weatherIcon["\\d\\d\\d\\d\\d\\d\\d"]    = { icon : 18 }; // not-available.png



//module functions

/*
 * some (unit conversion) helper methods
 */

function getTemperature(value) {
    return ( options.fahrenheit ? Math.round((value*9)/5 + 32) : Math.round(value) );
}

function getTemperatureUnit() {
    return ( options.fahrenheit ? "\u00B0F" : "\u00B0C" );
}

function getLength(value) {
    return ( options.inch ? Math.round(value*0.3937) : Math.round(value*10) );
}

function getLengthUnit() {
    return ( options.inch ? "in" : "mm" );
}

function getPressure(value) {
    switch (options.hpa) {
	    case 0:
	        return Math.round(value*0.750061683);
	        break;
	    default:
	        return Math.round(value);
	    	break;
    }
}

function getPressureUnit() {
    switch (options.hpa) {
	    case 0:
	        return "Torr";
	        break;
	    default:
	        return "hPa";
	    	break;
	}
}

function getSpeed(value) {
    switch (options.kns) {
	    case 0:
	        return Math.round(value*1.852);
	        break;
	    case 2:
	        return Math.round(value*1.151);
	        break;
	    default:
	        return Math.round(value);
	    	break;
    }
}

function getSpeedUnit() {
    switch (options.kns) {
	    case 0:
	        return "km/h";
	        break;
	    case 2:
	        return "mph";
	        break;
	    default:
	        return "kn";
	    	break;
	}
}

function getWeatherIcon(symbol) {
    var icon = 18;

    for(var key in weatherIcon) {
        if (key === 'length' || !weatherIcon.hasOwnProperty(key)) {
            continue;
        }

        var pattern = new RegExp(key);
        if(pattern.test(symbol)) {
            icon = weatherIcon[key].icon;
            break;
        }
    }

    return icon;
}

/*
 * fetch weather operations
 */

function fetchWeatherFromLID(lid, location) {
    // http://weatherpro.consumer.meteogroup.com/weatherpro/WeatherServiceFeed.php?lid=18225472&udid=000000000000000&format=json&mode=standard&version=WeatherPro4.8.8.4&lang=de
    var query = encodeURI("lid=" + lid);
    var url = "http://weatherpro.consumer.meteogroup.com/weatherpro/WeatherServiceFeed.php?" + query + "&udid=000000000000000&format=json&mode=standard&version=WeatherPro4.8.8.4&lang=de";

    var temperatureUnit = getTemperatureUnit();
    var lengthUnit      = getLengthUnit();
    var pressureUnit    = getPressureUnit();
    var speedUnit       = getSpeedUnit();

    appcontroller.startJSONRequest(url,
        function(result) {
        	var today     = new Date().setHours(0,0,0,0);

            // get current weather condition
            var condition = result.observation.obse.obs[0];

            // find first weather forecast
            var counter = 0;
            var forecast       = result.forecast.days.day[counter];
            var forecastDate   = Date.parse(forecast.dtg);

            while(typeof forecast != "undefined" && today>forecastDate){
                forecast = result.forecast.days.day[++counter];
                if (typeof forecast != "undefined") {
                    forecastDate  = Date.parse(forecast.dtg);
                }
            }

            appcontroller.pushToLocalStorageAndAppMessages(WEATHER_ID, {
                "message" : {
                    "weatherContent" :  getWeatherIcon(condition.symbol) + "|" +
					                    getTemperature(condition.tt) + temperatureUnit + "|" +
					                    getPressure(condition.ppp) + pressureUnit + "|" + // air pressure - Note: max temperature info is gone in actual json result
					                    getSpeed(condition.ff) + speedUnit + "|" + // windspeed - Note: min temperature info is gone in actual json result
					                    location + "|e"
                }
            });
        }
    );
}

function getWeatherFromGeoPosition(longitude, latitude) {
    // http://weatherpro.consumer.meteogroup.com/weatherpro/SearchFeed.php?next=9.496,47.658&lang=de&limit=20&udid=000000000000000&version=WeatherPro4.8.8.4
    var url = "http://weatherpro.consumer.meteogroup.com/weatherpro/SearchFeed.php?next=" + longitude + "," + latitude + "&lang=de&limit=20&udid=000000000000000&version=WeatherPro4.8.8.4";

    appcontroller.startXMLRequest(url,
        function(err, tmpResult) {
            var result = JSON.parse(JSON.stringify(tmpResult));
            // get first location out of the list
            if (result && result["result"] &&
                result["result"]["cities"]["length"] > 0 &&
                result["result"]["cities"][0]["city"]["length"] > 0 &&
                result["result"]["cities"][0]["city"][0]["$"]) {
                var city = result["result"]["cities"][0]["city"][0]["$"];

                var lid          = city.id;
                var locationName = city.name;

                //console.log("Success - got city for lat=" + latitude + ",longitude=" + longitude + "[name: " + locationName + ", lid: " + lid + "]");
                fetchWeatherFromLID(lid, locationName);
            }
            else {
                console.log("Error - found no location to provided geo position.");
                fetchWeatherFromLID(options.lid, options.locationName);
            }
        },
        function() {
            console.log("Error - found no location to provided geo position.");
            fetchWeatherFromLID(options.lid, options.locationName);
        }
    );
}

function fetchLocalWeather(mode) {
    if (options.fixLocation || !navigator.geolocation) {
        fetchWeatherFromLID(options.lid, options.locationName);
    }
    else {
        navigator.geolocation.getCurrentPosition(
            function(pos) {
                getWeatherFromGeoPosition(pos.coords.longitude, pos.coords.latitude);
            },
            function(err) {
                console.log("Error - can't locate! (" + err.code + "): " + err.message);
                fetchWeatherFromLID(options.lid, options.locationName);
            },
            {
                "enableHighAccuracy": parseInt(options.gpsaccuracy),
                "timeout": parseInt(options.gpstimeout) * 1000,
                "maximumAge": parseInt(options.gpsage) * 1000
            }
        );
    }
}



/*
 * individual module interace
 */

module.exports.fetch = fetchLocalWeather;
